<?php
/**
 * Cache class, GDocs Wordpress plugin
 *
 * Handles cache access
 *
 * @author Lim Jiunn Haur <fisherman.jiang@broken-watch.info>
 * @copyright Copyright (c) 2008, Lim Jiunn Haur
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @package gdocs
 * @since 0.5
 */

/**
 * GCache class
 *
 * All cache operations are carried out via this class.
 * It keeps a copy of the Google Doc/Spreadsheet in a simple html file
 * when the document is first displayed in a post or on a page. On
 * subsequent page requests, the document is loaded from the cache onto
 * the page, which is much faster than downloading the document data from
 * the Google server.
 * @subpackage inc
 */
class GCache {
	
	/**
	 * Cache directory
	 * @access private
	 * @var string
	 */
	private $dir;
	
	/**
	 * Name of HTML file to write to
	 * @access private
	 * @var string
	 */
	private $file;
	
	/**
	 * Class constructor
	 * @method GCache GCache ($doc_id) creates and initializes new GCache object (for Google Document)
	 * @method GCache GCache ($st_id, $wt_id) creates and initializes new GCache object (for Google Spreadsheet)
	 * @param $doc_id Google document ID
	 * @param $st_id Google Spreadsheet ID
	 * @param $wt_id Worksheet ID
	 * @return GCache object
	 * @access public
	 */
	public function __construct (){
		$this->dir = ABSPATH . 'wp-content/plugins/gdocs/cache/';
		$this->file = $this->getPath (func_get_args());
	}

	/**
	 * Checks if the doc/spreadsheet exists in cache. If so, retrieve.
	 * @access public
	 * @param $html passed by reference, used to store the HTML retrieved from cache
	 * @return boolean $cache_hit TRUE if cache hit, FALSE if miss
	 */
	public function read (&$html){
	
		// if HIT, return true, set $html
		if (file_exists ($this->file) && (time () - get_option ('gdocs_cache_expiry') * 60 < filemtime ($this->file))) {
			/*
			 * 1. Check if the file exists in cache
			 * 2. Check if file has expired
			 */
			$tbl = file_get_contents ($this->file);
			if ($tbl) {
				$html .= $tbl;	 
				return true;
			}
		
		}
		
		// if MISS, return false
		return false;
	
	}
	
	/**
	 * Check if caching is enabled. If so, create cache file.
	 * @access public
	 * @param $html contents of Google Doc/Spreadsheet, to be written to cache
	 */
	public function write ($html){
	
		if (get_option ('gdocs_cache_expiry')){
			$cacheFile = fopen ($this->file, 'wb');
			fwrite ($cacheFile, $html);
			fclose ($cacheFile);
		}
	
	}
	
	/**
	 * Assembles path to cache file
	 * @access private
	 * @param $atts array of ID's of Google Doc/Spreadsheet
	 * @return string $path path to the cache file
	 */
	private function getPath ($atts){
		switch (count($atts)){
			case 1:
				// Google Document
				return $this->dir . $this->cleanFilename ("doc_".$atts[0].".html");
				break;
			case 2:
				// Google Spreadsheet
				return $this->dir . $this->cleanFilename ("st_".$atts[0]."_".$atts[1].".html");
				break;
		}
	}
	
	/**
	 * Sanitizes filename
	 * @access private
	 * @param $filename raw filename
	 * @return string $filename clean filename
	 */
	private function cleanFilename ($filename){
		$reserved = preg_quote('\/:*?"<>|', '/');//characters that are  illegal on any of the 3 major OS's
		//replaces all characters up through space and all past ~ along with the above reserved characters
		return preg_replace("/([\\x7f-\\xff{$reserved}])/e", "_", $filename);
	}

	
}
?>