<?php
/**
 * Client class, GDocs Wordpress plugin
 * 
 * Handles authentication and proxy settings
 *
 * @author Lim Jiunn Haur <fisherman.jiang@broken-watch.info>
 * @copyright Copyright (c) 2008, Lim Jiunn Haur
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @package gdocs
 * @since 0.5
 */
 
/**
 * GClient class
 *
 * Uses singleton design pattern, so that not more than one
 * client exists at any one time.
 * @abstract
 * @subpackage inc
 */
abstract class GClient {

	/**
	 * Zend Gdata client
	 *
	 * This is used to query the Google servers. Connection
	 * settings are configured using the Zend Http Client.
	 * i.e. the GClient object contains the Zend_Gdata object,
	 * which contains the Zend_Http_client object
	 * @access protected
	 * @var mixed
	 */
	protected $client;
	
	/**
	 * Class constructor
	 *
	 * Creates authenticated Google client. Note that this is never called directly
	 * from public scope, but via the singleton function GClient_X::getInstance().
	 * @access protected
	 * @method GClient GClient ($service) creates and initializes new GClient object
	 * @param $service type of service access (either St, Doc, or Raw)
	 * @see GClient_Doc::getInstance()
	 * @see GClient_St::getInstance()
	 * @see GClient_Raw::getInstance()
	 */
	protected function __construct ($service){
		// get login credentials
		$user = get_option ('gdocs_user');
		$pwd = get_option ('gdocs_pwd');
		
		// create Zend Http Client
		$httpClient = Zend_Gdata_ClientLogin::getHttpClient($user, $pwd, $service);
		
		$this->setProxy ($httpClient);
		
		// create Zend Gdata client
		$this->client = new Zend_Gdata ($httpClient);
	}
	
	/**
	 * Retrieves feed from Google
	 * @access protected
	 * @param $url URL to query
	 * @return mixed $feed feed returned by Google
	 */
	protected function getFeed ($url){
		if (is_null($this->client)) throw new Exception ();
		return $this->client->getFeed ($url);
	}
	
	/**
	 * Configures proxy settings for the Zend Http Client
	 * @access protected
	 * @param $httpClient Zend Http Client
	 */
	protected function setProxy (&$httpClient){
	
		$proxy_host = get_option ('gdocs_proxy_host');
		
		// if user specified a proxy
		if ($proxy_host){
			// set proxy
			$httpClient->setConfig (array (
				'adapter' => 'Zend_Http_Client_Adapter_Proxy',
				'proxy_host' => $proxy_host,
				'proxy_port' => get_option ('gdocs_proxy_port'),
				'proxy_user' => get_option ('gdocs_proxy_user'),
				'proxy_pass' => get_option ('gdocs_proxy_pwd')
			));
		}
	
	}
	
	/**
	 * Factory function
	 *
	 * Gets corresponding class to check if a Gdata client already exists.
	 * @param $service type of service access (either St, Doc, or Raw)
	 * @return mixed $client GClient object
	 * @static
	 */
	public static function getInstance ($service){
		switch ($service){
			case 'St':
				// spreadsheet
				return GClient_St::getInstance ();
				break;
			case 'Doc':
				// document
				return GClient_Doc::getInstance ();
				break;
			case 'Raw':
				// raw
				return GClient_Raw::getInstance (func_get_arg(1));
				break;
			default;
				// safety net
				return NULL;
		}
	}

}

/**
 * GClient_Doc class
 * 
 * For Google Documents
 * @subpackage inc
 */
class GClient_Doc extends GClient {
	
	/**
	 * GClient_Doc object
	 * @static
	 * @access private
	 * @var mixed
	 */
	private static $_singleton;
	
	/**
	 * Class constructor
	 * @method GClient_Doc GClient_Doc () creates and initializes new GClient object for Google Documents
	 * @access protected
	 */
	protected function __construct (){
		parent::__construct (Zend_Gdata_Docs::AUTH_SERVICE_NAME);
	}
	
	public function getDocFeed (){
		return new GFeed ($this->getFeed ("http://docs.google.com/feeds/documents/private/full/-/document"), 'document');
	}
	
	public function getStFeed (){
		return new GFeed ($this->getFeed ("http://docs.google.com/feeds/documents/private/full/-/spreadsheet"), 'spreadsheet');
	}
	
	/**
	 * Singleton function
	 * @static
	 * @return mixed GClient_Doc object
	 */
	public static function getInstance (){
		if (is_null (self::$_singleton)){
			// if object not created yet, construct it
			self::$_singleton = new GClient_Doc ();
		}
		return self::$_singleton;
	}

}

/**
 * GClient_St class
 *
 * For Google Spreadsheets
 * @subpackage inc
 */
class GClient_St extends GClient {
	
	/**
	 * GClient_St object
	 * @static
	 * @access private
	 * @var mixed
	 */
	private static $_singleton;
	
	/**
	 * Class constructor
	 * @method GClient_St GClient_St () creates and initializes new GClient object for Google Spreadsheets
	 * @access protected
	 */
	protected function __construct (){
		parent::__construct (Zend_Gdata_Spreadsheets::AUTH_SERVICE_NAME);
	}
	
	public function getWtFeed ($st_id){
		return $this->getFeed ("http://spreadsheets.google.com/feeds/worksheets/{$st_id}/private/full");
	}
	
	public function getLtFeed ($st_id, $wt_id){
		return $this->getFeed ("http://spreadsheets.google.com/feeds/list/{$st_id}/{$wt_id}/private/full");
	}
	
	/**
	 * Singleton function
	 * @static
	 * @return mixed GClient_St object
	 */
	public static function getInstance (){
		if (is_null (self::$_singleton)){
			// if object not created yet, construct it
			self::$_singleton = new GClient_St ();
		}
		return self::$_singleton;
	}

	
}

/**
 * GClient_Raw class
 *
 * For raw http access.
 * Note: special case here. $this->client is not a Zend_Gdata object but Zend_Http_Client object
 * @subpackage inc
 */
class GClient_Raw extends GClient {

	/**
	 * GClient_Raw object
	 * @static
	 * @access private
	 * @var mixed
	 */
	private static $_singleton;
	
	/**
	 * Class constructor
	 * @method GClient_Raw GClient_Raw () creates and initializes new GClient object for raw http retrieval
	 * @access protected
	 */
	protected function __construct (){
		$httpClient = new Zend_Http_Client ('http://docs.google.com/RawDocContents');
		$this->setProxy ($httpClient);
		$this->client = $httpClient;
	}
	
	public function getFeed (){
		return $this->client->request()->getBody();
	}
	
	/**
	 * Singleton function
	 * @static
	 * @return mixed GClient_Raw object
	 * @param $atts attributes of Google document to retrieve
	 */
	public static function getInstance ($atts){
	
		if (is_null (self::$_singleton)){
			// if object not created yet, construct it
			self::$_singleton = new GClient_Raw ();
		}
				
		// set parameters
		self::$_singleton->client->setParameterGet (array (
			action => 'fetch',
			justBody => 'true',
			revision =>'_latest',
			editMode => 'false',
			docID => $atts['id']
		));
		
		return self::$_singleton;
	}

}

?>
